//-----------------------------------------------------------------------------
//  Copyright (C) 2014-2016 Thomas S. Ullrich
//
//  This file is part of "MineSweeper".
//
//  This file may be used under the terms of the GNU General Public License.
//  This project is free software; you can redistribute it and/or modify it
//  under the terms of the GNU General Public License.
//
//  Author: Thomas S. Ullrich
//  Last update: May 3, 2016
//-----------------------------------------------------------------------------
#include "msSettings.h"
#include "msVersion.h"
#include "msHighscoreList.h"
#include <QSettings>
#include <iostream>

using namespace std;

#define PR(x) cout << #x << " = " << (x) << endl;

unsigned int   msSettings::mNumXCells = 0;
unsigned int   msSettings::mNumYCells = 0;
unsigned int   msSettings::mNumberOfMines = 0;
unsigned int   msSettings::mMaxNumXCells = 0;
unsigned int   msSettings::mMaxNumYCells = 0;
unsigned int   msSettings::mCellSize = 0;
unsigned int   msSettings::mMinCellSize = 16;
unsigned int   msSettings::mMaxCellSize = 32;
unsigned int   msSettings::mNumberOfAllowedOopsies = 0;

int msSettings::mMaxHighscoreItems = 10;

bool msSettings::mEasyStart = true;
bool msSettings::mAllowQuestionableFlag = true;

QColor msSettings::mWaterColor = QColor();
QColor msSettings::mBackgroundColor = QColor();

QColor msSettings::mWaterColorDefault = QColor(0, 128, 168);      // turquoise
QColor msSettings::mBackgroundColorDefault = QColor(225,225,225); // light gray
unsigned int msSettings::mCellSizeDefault = 30;

msSettings::msMineType msSettings::mCurrentMineType = msMineType::mine1;
msSettings::msFlagType msSettings::mCurrentFlagType = msFlagType::flag1;
msSettings::msGameSize msSettings::mCurrentGameSize = msGameSize::medium;

unsigned int msSettings::mSmallSizeParameters[3] = { 10, 10, 20 }; // columns, rows, mines
unsigned int msSettings::mMediumSizeParameters[3] = {20, 12, 48 };
unsigned int msSettings::mLargeSizeParameters[3] = {30, 20, 120};

QString msSettings::mLastPlayer = QString("My Name");

msSettings::msSettings()
{
    mSettings = new QSettings(QSettings::NativeFormat, QSettings::UserScope, "tu", "MineSweeper");
    mSettings->setFallbacksEnabled(false);
    mHighscoreList = 0;
}

msSettings::~msSettings() { /* no op */}

void msSettings::load()
{
    mSettings->beginGroup("/MineSweeper");
    mSettings->beginGroup("/Preferences");
    mNumXCells = mSettings->value("/NumXCells", msSettings::mMediumSizeParameters[0]).toUInt();
    mNumYCells = mSettings->value("/NumYCells", msSettings::mMediumSizeParameters[1]).toUInt();
    mNumberOfMines = mSettings->value("/NumberOfMines", msSettings::mMediumSizeParameters[2]).toUInt();
    mCellSize = mSettings->value("/CellSize", mCellSizeDefault).toUInt();
    mEasyStart = mSettings->value("/EasyStart", true).toBool();
    mAllowQuestionableFlag = mSettings->value("/QuestionableFlag", true).toBool();
    mNumberOfAllowedOopsies= mSettings->value("/AllowedOopsies", 0).toInt();
    int r = mSettings->value("/WaterColor/red", msSettings::mWaterColorDefault.red()).toInt();
    int g = mSettings->value("/WaterColor/green", msSettings::mWaterColorDefault.green()).toInt();
    int b = mSettings->value("/WaterColor/blue", msSettings::mWaterColorDefault.blue()).toInt();
    mWaterColor = QColor(r, g, b);
    r = mSettings->value("/BackgroundColor/red", msSettings::mBackgroundColorDefault.red()).toInt();
    g = mSettings->value("/BackgroundColor/green", msSettings::mBackgroundColorDefault.green()).toInt();
    b = mSettings->value("/BackgroundColor/blue", msSettings::mBackgroundColorDefault.blue()).toInt();
    mBackgroundColor = QColor(r, g, b);
    mCurrentMineType = static_cast<msMineType>(mSettings->value("/MineType", static_cast<int>(msMineType::mine1)).toInt());
    mCurrentFlagType = static_cast<msFlagType>(mSettings->value("/FlagType", static_cast<int>(msFlagType::flag1)).toInt());
    mCurrentGameSize = static_cast<msGameSize>(mSettings->value("/GameSize", static_cast<int>(msGameSize::medium)).toInt());
    mLastPlayer = mSettings->value("/LastPlayer", "My Name").toString();
    mSettings->endGroup();
    if (mHighscoreList) mHighscoreList->load(mSettings);
    mSettings->endGroup();
}

void msSettings::save()
{
    mSettings->beginGroup("/MineSweeper");
    mSettings->setValue("/Version", VERSION_NUMBER);  // ignored so far (but good to have it)
    mSettings->beginGroup("/Preferences");
    mSettings->setValue("/NumXCells", mNumXCells);
    mSettings->setValue("/NumYCells", mNumYCells);
    mSettings->setValue("/NumberOfMines", mNumberOfMines); 
    mSettings->setValue("/CellSize", mCellSize);
    mSettings->setValue("/EasyStart", mEasyStart);
    mSettings->setValue("/QuestionableFlag", mAllowQuestionableFlag);
    mSettings->setValue("/AllowedOopsies", mNumberOfAllowedOopsies);
    mSettings->setValue("/WaterColor/red", mWaterColor.red());
    mSettings->setValue("/WaterColor/green", mWaterColor.green());
    mSettings->setValue("/WaterColor/blue", mWaterColor.blue());
    mSettings->setValue("/BackgroundColor/red", mBackgroundColor.red());
    mSettings->setValue("/BackgroundColor/green", mBackgroundColor.green());
    mSettings->setValue("/BackgroundColor/blue", mBackgroundColor.blue());
    mSettings->setValue("/MineType", static_cast<int>(mCurrentMineType));
    mSettings->setValue("/FlagType", static_cast<int>(mCurrentFlagType));
    mSettings->setValue("/GameSize", static_cast<int>(mCurrentGameSize));
    mSettings->setValue("/LastPlayer", mLastPlayer);
    mSettings->endGroup();
    if (mHighscoreList)  mHighscoreList->save(mSettings);
    mSettings->endGroup();
    
    mSettings->sync();
}

unsigned int msSettings::numberOfCells() {return mNumXCells*mNumYCells;}

QString msSettings::mineFileName()
{
    QString str = ":/images/";
    switch (mCurrentMineType) {
        case msMineType::mine1:
            str += "mine1.svg";
            break;
        case msMineType::mine2:
            str += "mine2.svg";
            break;
        case msMineType::mine3:
            str += "mine3.svg";
            break;
        case msMineType::bomb1:
            str += "bomb1.svg";
            break;
        case msMineType::bomb2:
            str += "bomb2.svg";
            break;
        case msMineType::bomb3:
            str += "bomb3.svg";
            break;
        default:
            break;
    }
    return str;
}

QString msSettings::flagFileName()
{
    QString str = ":/images/";
    switch (mCurrentFlagType) {
        case msFlagType::flag1:
            str += "flag1.svg";
            break;
        case msFlagType::flag2:
            str += "flag2.svg";
            break;
        case msFlagType::flag3:
            str += "flag3.svg";
            break;
        case msFlagType::warning1:
            str += "warning1.svg";
            break;
        case msFlagType::warning2:
            str += "warning2.svg";
            break;
        case msFlagType::warning3:
            str += "warning3.svg";
            break;
        default:
            break;
    }
    return str;
}

QString msSettings::tryOutSymbolFileName()
{
    return QString(":/images/question_mark.svg");
}

QString msSettings::numberFileName(int i)
{
    QString str = ":/images/";
    switch (i) {
        case 1:
            str += "one.svg";
            break;
        case 2:
            str += "two.svg";
            break;
        case 3:
            str += "three.svg";
            break;
        case 4:
            str += "four.svg";
            break;
        case 5:
            str += "five.svg";
            break;
        case 6:
            str += "six.svg";
            break;
        case 7:
            str += "seven.svg";
            break;
        case 8:
            str += "eight.svg";
            break;
        default:
            str = "";
            break;
    }
    return str;
   
}

QSettings* msSettings::settings() {return mSettings;}

void msSettings::setHighscoreList(msHighscoreList* list)
{
    mHighscoreList = list;
}
